<?php

declare(strict_types=1);

namespace Drupal\user_entity_reference_selection\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\Attribute\EntityReferenceSelection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user_entity_reference_selection\Plugin\Derivative\UserEntitySelectionDeriver;
use Psr\Log\LoggerInterface;

/**
 * Provides the base implementation of user entity reference selection.
 *
 * @todo Expand to cover other entity types.
 */
#[EntityReferenceSelection(
  id: "user_entity_reference_selection",
  label: new TranslatableMarkup("Default with fixed user access"),
  group: "user_entity_reference_selection",
  weight: 0,
  deriver: UserEntitySelectionDeriver::class,
)]
class UserEntitySelection extends DefaultSelection {

  use UserEntitySelectionTrait;

  /**
   * Constructs a new user entity selection object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\Core\Render\ElementInfoManagerInterface $element_info_manager
   *   The element information manager.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    mixed $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleHandlerInterface $module_handler,
    AccountInterface $current_user,
    EntityFieldManagerInterface $entity_field_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    EntityRepositoryInterface $entity_repository,
    ElementInfoManagerInterface $element_info_manager,
    LoggerInterface $logger,
  ) {
    $this->setElementInfo($element_info_manager);
    $this->setLogger($logger);
    $user = $this->getUserFromConfig($configuration, $entity_repository) ?? $current_user;
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $module_handler, $user, $entity_field_manager, $entity_type_bundle_info, $entity_repository);
  }

}
